# Auslesen der Eingänge am intelligent Interface 30402
# Einlesen der Analogwerte von EY und Ausgabe als Prozentsatz
# zusätzlich können beliebige digitale Eingänge abgefragt werden
#
# Zu Beginn ist Lampe(M4) aus und Motor(M1) aus.
# es wird die Photozelle an EY abgefragt. Die Photozelle hängt
# parallen an einem Taster, der als Brücke funktioniert und
# bei Betätigung 100% an EY liefert.
# Der Wert der Photozelle wird nur bei Änderung ausgegeben.
# Ist der analoge Wert der Photozelle größer als 60% wird die
# Lampe an M4 und der Motor an M1 eingeschaltet.
# Wird in diesem Zustand zusätzlich Taster E2 gedrückt, wird die
# Drehrichtung des Motors M1 geändert.
# Der Taster E1 gibt nur eine Meldung aus.
# Der Taster E8 ist ein Reset-Taster und stoppt die Programmausführung
#
# import von Funktionen
from binascii import unhexlify
import serial

# serielle Schnittstelle definieren
ser = serial.Serial('/dev/ttyS0', 9600)

# Empfang der seriellen Zustands-Info in diese Variable
data = [0,1,2]
einbyte = ''

#Programmstart mit Info
print('fischertechnik intelligent interface 30402')
print('seriell-Protokoll untersuchen')
print('Peter Fürle, www.nc-x.com, Jan2021')

def callInterface(typ):
    # welche Werte vom Interface lesen ?
    if   (typ=='EX'):
        interface = b'11000101' # 3 bytes für E1-E8 und EX Analogwert einlesen
    elif (typ=='EY'):
        interface = b'11001001' # 3 bytes für E1-E8 und EY Analogwert einlesen
    else:
        interface = b'11000001' # nur 1 byte für E1-E8 Zustand einlesen
    ea_zustand=('%02X' % int(interface,2))
    # byte für Ausgabe erzeugen. 
    return(unhexlify(ea_zustand))


# Startsituation für Motoren, Motor 1 ein
# M4 M3 M2 M1
# 00 00 00 00
def MotorenStartsituation():
    motorzustand='00000010' # Motor M4 ein
    return(motorzustand)

# Funktion prüft ob Eingangs-Signal anliegt
def checkEingang(vonSeriell,nr):
    # es wird das seriell eingelesene byte übergeben
    # es wird der abzufragende Eingang 1-8 übergeben
    vonSeriell=('{:08b}'.format(int.from_bytes(vonSeriell, 'big', signed=False)))
    # sooft links shiften bis Eingang nr ereicht ist
    e=(1 << nr-1)
    # 8stellige binärmaske erzeugen
    bitmaske_8=(bin(e & (1<<8) - 1)[2:].zfill(8))
    # binären Operator UND auf serielles byte und Bitmaske anwenden
    if (int(vonSeriell,2) & int(bitmaske_8,2)):
        #print('Signal erkannt!')
        return(True)

def motor2byte(mot):
    #Motorzustand von binär nach hex formatieren
    motoren=('%02X' % int(mot,2))
    #Motorzustand als Byte 
    motoren=unhexlify(motoren)
    return(motoren)


def MotorenAus():
    motorzustand='00000000'
    return(motorzustand)

def checkAnalog():
    # Zustand von E1-E8 und EX/EY analog einlesen
    # für stabilen Wert 10* einlesen
    for z in range(1,10):
        # sicherheitshalber Daten über seriell senden
        ser.write(command)
        # 3 bytes einlesen
        for i in range(0,3):
            data[i] = ser.read()
        # data[0] ist der E1-E8 Zustand
        # data[1] und data[2] sind der Analogwert von EX/EY
        serbytes = data[1] + data[2]
        analog_in = int.from_bytes(serbytes, byteorder='big')
        proz=100-(100/1024*analog_in)
    #print('Analogwert ist bei',int(proz),'%')
    return(int(proz))

def checkDigital_Drei():
    # Zustand von E/A und EY einlesen, nur 1. byte verwenden.
    # sicherheitshalber Daten über seriell senden
    ser.write(command)
    # 3 bytes einlesen
    for i in range(0,3):
        data[i] = ser.read()
    # data[0] ist der E/A Zustand
    # data[1] und data[2] sind der Analogwert
    return(data[0])  

# wird in der Motorsteuerung verwendet um alte Zustände zu löschen
def clear_bit(value, bit):
    return value & ~(1<<bit)

# ein weiterer Motor wird in das Motorregister eingestellt
def einzelMotor(m,mot,mo_akt):
    # Aufruf mit m=Motornummer(1-4) und mot=links/rechts
    #print('Anforderung Motor #',m,' Richtung:',mot)#, 'mo_alt:',mo_akt)
    # die bits, die neu besetzt werden im alten Zustand löschen
    # damit nicht eventuell beide Richtungen für einen Motor ein sind.
    mo_alt=int(mo_akt,2)
    # beide Motorbits hintereinander auf 0 setzen
    mo_alt=(clear_bit(mo_alt,(m*2)-1))
    mo_alt=(clear_bit(mo_alt,(m*2)-2))
    # oder-Verknüpfung bereinigter Alt-Zustand mit neuer Maske
    mo_neu=bin(int(mot,2)<<(m-1)*2 & (1<<8) - 1)[2:].zfill(8)
    mo_akt=mo_alt | int(mo_neu,2)
    #print('Zustand des Motorregisters nach Anforderung:',bin(mo_akt & (1<<8) - 1)[2:].zfill(8))
    return(bin(mo_akt & (1<<8) - 1)[2:].zfill(8))


#Hauptprogramm

#Vorbelegungen
motorzustand=MotorenAus()
potentiometer_alt=100
potentiometer=0
links='01'
rechts='10'

ea_zustand=callInterface('EY') # analog EY und digitale Signale abfragen

while True: 
    #beide bytes zusammensetzen für serielle Ausgabe
    command = ea_zustand + motor2byte(motorzustand)
    ser.write(command)
    # Zustand der Ein-Ausgänge einlesen
    potentiometer=checkAnalog()
    # Potentiometer-Wert nur ausgeben bei Änderung
    if potentiometer != potentiometer_alt:
        print('Potentiometer steht bei',potentiometer,'%')
        potentiometer_alt=potentiometer
    # E1 Taster schickt M2 hoch
    if(checkEingang(checkDigital_Drei(),1)):
        print('Eingang 1 ist aktiv')
        motorzustand=(einzelMotor(2,rechts,motorzustand))
    else:
        motorzustand=(einzelMotor(2,'00',motorzustand)) # nur M2 ausschalten
    # EY analogwert mit Brücke durch Taster E2
    if (potentiometer >= 60):
        if(checkEingang(checkDigital_Drei(),2)):
            print('Eingang 2 ändert Drehrichtung M1')
            motorzustand=(einzelMotor(1,rechts,motorzustand)) # M1 Ventilator an
            motorzustand=(einzelMotor(4,rechts,motorzustand)) # M4 Signallampe an
        else:
            motorzustand=(einzelMotor(1,links,motorzustand))  # M1 Ventilator Richtungsumkehr
            motorzustand=(einzelMotor(4,rechts,motorzustand)) # M4 Signallampe an
    else:
        motorzustand=(einzelMotor(1,'00',motorzustand)) # M1 Ventilator aus
        motorzustand=(einzelMotor(4,'00',motorzustand)) # M4 Signallampe aus
    # Reset-Taster E8
    if(checkEingang(checkDigital_Drei(),8)):
        print('Reset über E8 aktiviert')
        break
# Ende While-Schleife

#informativ
print('Kommunikation mit Inferface beendet')
print('Vielen Dank für Ihre Mitwirkung')

